//===========================================================================
//
//      hal_priv.c
//
//      SPARC Architecture LEON-specific private variables
//
//===========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//===========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    hmt
// Contributors: hmt
// Date:         1999-02-20
// Purpose:      private vars for LEON processor.
//              
//####DESCRIPTIONEND####
//
//===========================================================================


#include <cyg/hal/hal_intr.h>
#include <cyg/hal/hal_arch.h>

// ------------------------------------------------------------------------
// Clock static to keep period recorded.
cyg_int32 cyg_hal_sparc_clock_period = 0;

// ------------------------------------------------------------------------
// Board specific startups.

extern void hal_board_prestart( void );
extern void hal_board_poststart( void );

void hal_board_prestart( void )
{
}

void hal_board_poststart( void )
{
    HAL_ENABLE_INTERRUPTS();
    // OK to do this post constructors, and good for testing.
}

cyg_uint32
hal_lsbit_index(cyg_uint32 mask)
{
    int i;
    for (i = 0;  i < 32;  i++) {
    if (mask & (1<<i)) return ((cyg_uint32)i);
    }
    return ((cyg_uint32)-1);
}

cyg_uint32
hal_msbit_index(cyg_uint32 mask)
{
    int i;
    for (i = 31;  i >= 0;  i--) {
        if (mask & (1<<i)) return ((cyg_uint32)i);
    }
    return ((cyg_uint32)-1);
}

void
hal_idle_thread_action(cyg_uint32 loop_count)
{
    cyg_uint32 old;

    // We are about to enter sleep mode. Disable interrupts so we can not be interrupted
    // during the process.
    HAL_DISABLE_INTERRUPTS(old);

    // "sethi  %%hi(0x80000000), %%g1;"   Setup to clear the power down mode regsiter
    //    "or     %%g1, 0x18, %%g2;"      
    //    "clr    [%%g2];"                Clear the power down mode register. This sets
    //                                    up power down mode so on the next load or store
    //                                    it will be activated.
    //    "rd     %%psr, %%l7;"           Read the current processor status register. 
    //    "andn   %%l7, 0xf20, %%l7;"     Mask the interrupt bits (PIL)
    //    "and    %0, 0xf20, %%l6;"       Restore the old pil from "old" ..see HAL_DISABLE above
    //    "wr     %%l6, %%l7, %%psr;"     Write this back to processor status register
    //    "nop; nop; ld [%%g2], %%g2;"    2 Nops to attempt to make the sleep mode activation (the ld) and
    //                                    the re-enabling of interrupts (the wr) to occur at the same time. 

    
    // Here's the actual code to do the above commented block.......
    asm volatile (                              \   
        "sethi  %%hi(0x80000000), %%g1;"        \
        "or     %%g1, 0x18, %%g2;"              \
        "clr    [%%g2];"                        \
        "rd     %%psr, %%l7;"                   \
        "andn   %%l7, 0xf20, %%l7;"             \
        "and    %0, 0xf20, %%l6;"               \
        "wr     %%l6, %%l7, %%psr;"             \
        "nop; nop; ld [%%g2], %%g2;"            \
        :                                       \
        : "r"(old)                              \
        : "l6","l7","g1","g2"                   \
        );    
}

// EOF hal_priv.c
